import { ComponentFixture, TestBed, waitForAsync, fakeAsync, tick } from '@angular/core/testing';
import { GitsearchTestModule } from '../../test.module';
import { PagesComponent } from 'app/editorialPages/pages.component';
import { PagesService } from 'app/shared/service/pages-service';
import { ActivatedRoute, Router, RouterState } from '@angular/router';
import { PagesMarkDownViewerComponent } from 'app/editorialPages/markDownViewer/markDownViewer.component';
import { MarkdownModule, MarkdownService } from 'ngx-markdown';
import { RouterTestingModule } from '@angular/router/testing';
import { PagesResolve } from 'app/editorialPages/pages.route';
import { Location } from '@angular/common';
import { of, Subject } from 'rxjs';
import { By } from '@angular/platform-browser';
import { TranslateService } from '@ngx-translate/core';

describe('Component Tests', () => {
    describe('Pages Component', () => {
        let comp: PagesComponent;
        let fixture: ComponentFixture<PagesComponent>;
        let compMD: PagesMarkDownViewerComponent;
        let fixtureMD: ComponentFixture<PagesMarkDownViewerComponent>;
        let pagesService: PagesService;
        let router: Router;
        let route: ActivatedRoute;
        let location: Location;

        beforeEach(waitForAsync(() => {

            const pagesTestRoutes = {
                path: 'pages',
                children: [
                  {
                    path: "**",
                    // loadChildren: () => import('app/editorialPages/editorialPages.module').then(m => m.PagesModule),
                    resolve: {
                      pagePath: PagesResolve,
                    },
                    component: PagesComponent,
                  }
                ]
              }
            TestBed.configureTestingModule({
                imports: [GitsearchTestModule, MarkdownModule.forRoot(), RouterTestingModule.withRoutes([pagesTestRoutes])],
                declarations: [PagesComponent, PagesMarkDownViewerComponent],
                    providers: [
                      MarkdownService,
                      {
                        provide: ActivatedRoute,
                        useValue: 
                           {url: of([]), data: of({pagePath: 'start'}), params: of({id: 123}), queryParams:of([]), outlet:"Hello World", 
                           component: "1", tagged: "hello WORLD"}
                      }
                      ],
                   })
                .overrideTemplate(PagesComponent, '')
                .overrideTemplate(PagesMarkDownViewerComponent, '')
                .compileComponents();
        }));

        beforeEach(() => {
            fixture = TestBed.createComponent(PagesComponent);
            comp = fixture.componentInstance;
            fixtureMD = TestBed.createComponent(PagesMarkDownViewerComponent);
            compMD = fixtureMD.componentInstance;
            router = TestBed.get(Router);
            route = TestBed.get(ActivatedRoute);
            location = TestBed.get(Location);
//            router = fixture.debugElement.injector.get(Router);
            router.initialNavigation();
            pagesService = fixture.debugElement.injector.get(PagesService);
        });

        it('check routing of pages component', fakeAsync(() => {
          /** unfortunatelly very rudimentary :-( 
           * router.navigate does not trigger page observers
           */
            // GIVEN

            const translateService = fixture.debugElement.injector.get(TranslateService);
            translateService.currentLang = "de";

            comp.ngOnInit();
            compMD.ngOnInit();
            
            router.navigate(['pages/de/start']);
            
            const getPageSpy = spyOn(pagesService,'getPage').and.callFake((pagePath: string) => {
               return of(  {path: pagePath, content: "#head \n hello world"});
            });
            
            tick();
            // WHEN, 
            compMD.path = "/start";

            fixtureMD.detectChanges();

            // THEN
            expect(location.path()).toBe("/pages/de/start");
            expect(comp.pagePath).toBe("start");
            expect(compMD.page.path).toBe('de/start');
        }));

  });
})
